use std::io;

pub mod reader;
pub mod writer;

pub use reader::TextReader;
pub use writer::TextWriter;

use crate::BytesBinarySource;
use crate::Domain;
use crate::DomainDecode;
use crate::IOValue;
use crate::Reader;
use crate::Value;

pub fn from_str<'de, D: Domain + 'static, Dec: DomainDecode<D>>(
    s: &'de str,
    decode_embedded: &mut Dec,
) -> io::Result<Box<dyn Value<D>>> {
    BytesBinarySource::new(s.as_bytes()).text().next(false, decode_embedded)
}

pub fn iovalue_from_str(s: &str) -> io::Result<IOValue> {
    BytesBinarySource::new(s.as_bytes()).text().next_iovalue(false)
}

pub fn annotated_from_str<'de, D: Domain + 'static, Dec: DomainDecode<D>>(
    s: &'de str,
    decode_embedded: &mut Dec,
) -> io::Result<Box<dyn Value<D>>> {
    BytesBinarySource::new(s.as_bytes()).text().next(true, decode_embedded)
}

pub fn annotated_iovalue_from_str(s: &str) -> io::Result<IOValue> {
    BytesBinarySource::new(s.as_bytes()).text().next_iovalue(true)
}
