use num::bigint::BigInt;
use std::io;

use crate::SignedInteger;
use crate::boundary as B;
use crate::signed_integer::SignedIntegerRepr;

pub trait Writer {
    fn boundary(&mut self, b: &B::Type) -> io::Result<()>;

    fn start_annotations(&mut self) -> io::Result<()>;
    fn end_annotations(&mut self) -> io::Result<()>;

    fn write_bool(&mut self, v: bool) -> io::Result<()>;

    fn write_f32(&mut self, v: f32) -> io::Result<()>;
    fn write_f64(&mut self, v: f64) -> io::Result<()>;

    fn write_i128(&mut self, v: i128) -> io::Result<()>;
    fn write_u128(&mut self, v: u128) -> io::Result<()>;
    fn write_int(&mut self, v: &BigInt) -> io::Result<()>;

    fn write_string(&mut self, v: &str) -> io::Result<()>;
    fn write_bytes(&mut self, v: &[u8]) -> io::Result<()>;
    fn write_symbol(&mut self, v: &str) -> io::Result<()>;

    fn start_record(&mut self) -> io::Result<()>;
    fn end_record(&mut self) -> io::Result<()>;

    fn start_sequence(&mut self) -> io::Result<()>;
    fn end_sequence(&mut self) -> io::Result<()>;

    fn start_set(&mut self) -> io::Result<()>;
    fn end_set(&mut self) -> io::Result<()>;

    fn start_dictionary(&mut self) -> io::Result<()>;
    fn end_dictionary(&mut self) -> io::Result<()>;

    fn start_embedded(&mut self) -> io::Result<()>;
    fn end_embedded(&mut self) -> io::Result<()>;

    fn flush(&mut self) -> io::Result<()>;

    //---------------------------------------------------------------------------

    fn write_i8(&mut self, v: i8) -> io::Result<()> { self.write_i128(v as i128) }
    fn write_u8(&mut self, v: u8) -> io::Result<()> { self.write_u128(v as u128) }
    fn write_i16(&mut self, v: i16) -> io::Result<()> { self.write_i128(v as i128) }
    fn write_u16(&mut self, v: u16) -> io::Result<()> { self.write_u128(v as u128) }
    fn write_i32(&mut self, v: i32) -> io::Result<()> { self.write_i128(v as i128) }
    fn write_u32(&mut self, v: u32) -> io::Result<()> { self.write_u128(v as u128) }
    fn write_i64(&mut self, v: i64) -> io::Result<()> { self.write_i128(v as i128) }
    fn write_u64(&mut self, v: u64) -> io::Result<()> { self.write_u128(v as u128) }

    fn write_signed_integer(&mut self, v: &SignedInteger) -> io::Result<()> {
        match v.repr() {
            SignedIntegerRepr::I128(i) => self.write_i128(*i),
            SignedIntegerRepr::U128(u) => self.write_u128(*u),
            SignedIntegerRepr::Big(n) => self.write_int(n),
        }
    }
}
